const mongoose = require("mongoose");

const { Schema } = mongoose;

//Schema que gestiona els punts de geoposicionament
const unitatSchema = new Schema({
    geometry: {
        type: {
            type: String,
            default: "Point",
        },
        coordinates: {
            type: [Number],
            required: true,
        },
    },
    properties: {
        imei: { type: String, required: true },
        type: { type: String, default: "" },
        event_id: { type: Number, default: 0 },
        timestamp: {
            type: Date,
            default: Date.now,
        },
        gps: {
            altitude: { type: Number, default: 0 },
            angle: { type: Number, default: 0 },
            speed: { type: Number, default: 0 },
            satellites: { type: Number, default: 0 },
            precision: { type: Number, default: 0},
        },
        ioElements: {
            ignition: { type: Boolean, default: true },
            gsm_strengh: { type: Number, default: 0 },
            fuel: { type: Number, default: 0 },
            battery: { type: Number, default: 0 },
            battery_device: { type: Number, default: 0 },
        },
    },
});

const Unitat = mongoose.model("Unitat", unitatSchema);

//Crea una unitat indiferentment si les dades son 
//de dispositiu o de telefon

const getUnitByData = (dades) => {
    if(!dades?.precision ){
        dades.precision =0;
    }
    const unitat = {
        geometry: {
            coordinates: [
                Number(dades?.gps?.longitude ?? dades?.lon).toFixed(6),
                Number(dades?.gps?.latitude ?? dades?.lat).toFixed(6),
            ],
        },
        properties: {
            imei: dades.imei,
            type: "device",
            timestamp: dades?.time ?? dades?.timestamp,
            event_id: dades?.gps?.event_id ?? 0,
            gps: {
                altitude:
                    dades?.gps?.altitude ?? Number(dades?.altitude).toFixed(),
                angle: dades?.gps?.angle ?? Number(dades?.angle).toFixed(),
                speed: dades?.gps?.speed ?? Number(dades?.speed).toFixed(),
                satellites:
                    dades?.gps?.satellites ??
                    Number(dades?.satellites).toFixed(),
                precision: dades?.precision ?? Number(dades?.precision).toFixed(),
            },
            ioElements: {
                battery_device: Number(dades?.battery_device).toFixed(),
            },
        },
    };
    if (
        dades.hasOwnProperty("ioElements") &&
        Array.isArray(dades?.ioElements) &&
        dades.ioElements.length > 0
    ) {
        for (let param of dades.ioElements) {
            unitat.properties.type = "dongle";
            switch (param.id) {
                case 239:
                    unitat.properties.ioElements.ignition =
                        param.value == 1 ? true : false;
                    break;

                case 21:
                    unitat.properties.ioElements.gsm_strengh = param.value;
                    break;

                case 48:
                    unitat.properties.ioElements.fuel = param.value;
                    break;

                case 66:
                    unitat.properties.ioElements.battery = param.value;
                    break;

                case 67:
                    unitat.properties.ioElements.battery_device = param.value;
                    break;
            }
        }
    }
    return unitat;
};

//Index per fer les cerques per imei i per temps
unitatSchema.index({ "properties.imei": 1, "properties.timestamp": 1 });

module.exports = {
    Unitat: mongoose.models.Unitat || mongoose.model("Unitat", unitatSchema),
    getUnitByData,
};
