/**
 * Servidor que gestiona els enviaments de dades dels
 * dispositius mobils.
 */

const { enviarLog } = require("../models/log_server");
const { Unitat, getUnitByData } = require("../models/unitat");
const { Object } = require("../models/object");
const express = require("express");
const cors = require("cors"); 

//SSL
const https = require("https"); 
const fs = require("fs"); 
const privateKey = fs.readFileSync("sslcert/server_phones/key.pem", "utf8");
const certificate = fs.readFileSync("sslcert/server_phones/cert.pem", "utf8");
const credentials = { key: privateKey, cert: certificate };
 
class Server {
    constructor() {
        this.app = express();
        this.httpsServer = https.createServer(credentials, this.app);
        this.port = process.env.PORT_PHONES;

        // Middlewares
        this.middlewares();

        // // Rutas de mi aplicación
        this.routes();
    }

    middlewares() {
        // CORS
        let corsOptions = {
            methods: ["POST"],
            optionsSuccessStatus: 200,
            credentials: true,
            allowedHeaders: ["Content-Type", "ApiToken"],
        };

        this.app.use(cors(corsOptions));
        // Lectura i parseig del body
        this.app.use(express.json());
        this.app.use(
            express.urlencoded({
                extended: true,
            })
        );
    }

    routes() {
        try {
            this.app.post("/", async (req, res) => {
                const dades = req?.body;
                const headers = req?.headers;

                //Check que sigui un objecte i tingui alguns
                //dels camps requerits
                if (
                    dades &&
                    typeof dades == "object" &&
                    dades.hasOwnProperty("lon") &&
                    headers.hasOwnProperty("apitoken")
                ) {
                    const unitat = new Unitat(getUnitByData(dades));

                    //Cercar objecte a partir del imei
                    const object = await Object.findOne({
                        IMEI: dades.imei,
                    }).lean();
                    //Comparar el apitoken
                    if (object && object.apitoken === headers.apitoken) {
                        //Gravar si coincideixen.
                        unitat
                            .save()
                            .then(() => {
                                console.info(
                                    ` - NOM: ${dades.tab}, IMEI: ${
                                        unitat.properties.imei
                                    }, ID:  ${
                                        unitat.properties.event_id
                                    }, HORA: ${new Date(
                                        unitat.properties.timestamp
                                    ).toLocaleString("ca-ES", {
                                        timeZone: "Europe/Madrid",
                                    })}, COOR (Lat/Long): ${
                                        unitat.geometry.coordinates[1]
                                    } ${unitat.geometry.coordinates[0]}`
                                );
                                res.status(200).json({ result: true });
                            }) //OK
                            .catch((error) => {
                                res.status(500).json({
                                    error,
                                    result: false,
                                }); //Error server
                                console.log(error);
                            });
                    } else {
                        res.status(401).json({ result: false }); //Unauthorized
                        console.log("NO AUTORITZAT -- No coincideix el token");
                    }
                } else {
                    res.status(406).json({ result: false }); //Not acceptable
                }
            });
        } catch (e) {
            console.error(e);
            enviarLog({
                type: "error",
                source: "servidor_express",
                message: JSON.stringify(e),
            });
        }
    }

    listen() {
        this.httpsServer.listen(this.port, () => {
            console.log(`Servidor HTTPS escoltant al port ${this.port}`);
        });
    }
}
module.exports = Server;
