//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

const { enviarLog } = require("../models/log_server");
const Parser = require("teltonika-parser-extended");
const net = require("net");
const { getUnitByData, Unitat } = require("../models/unitat");
const binutils = require("binutils64");



class Server {
    constructor() {
        try {
            this.server = net.createServer();

            this.port = process.env.PORT_DEVICES;

            // Middlewares
            this.middlewares();

            // // Rutas de mi aplicación
            this.routes();
        } catch (e) {
            console.error(e);
            enviarLog({
                type: "error",
                source: "constructor_server_dongle",
                message: JSON.stringify(e),
            });
        }
    }

    middlewares() {
        // CORS
    }

    listen() {
        this.server.listen(this.port, () => {
            console.log(
                `Servidor Teltonika HTTPS escoltant al port ${this.port}`
            );
        });
    }

    routes() {
        this.server.on("connection", async (socket) => {
            let imei;
            console.log("Client connectat");
            console.log(
                "Ip del client: " +
                    socket.remoteAddress +
                    ":" +
                    socket.remotePort
            );
            socket.on("data", async (data) => {                
                // console.log(`Llegint dades del GPS`);
                let buffer = data;
                let parser = new Parser(buffer);                   
                if (parser.isImei) {
                    socket.write(Buffer.alloc(1, 1));
                    imei = parser.imei;
                    console.log(`ENTRADA IMEI: ${parser.imei}`);
                } else {
                    let avl = parser.getAvl();
                    let writer = new binutils.BinaryWriter();
                    writer.WriteInt32(avl.number_of_data);

                    let response = writer.ByteBuffer;
                    socket.write(response);

                    // TODO parser
                    let ar_Records = [];
                    if (
                        avl.hasOwnProperty("records") &&
                        Array.isArray(avl?.records) &&
                        avl.records.length > 0
                    ) {
                        for (let d of avl.records) {
                            d.imei = imei;
                            ar_Records.push(getUnitByData(d));
                        }
                    }
                    await Unitat.init();
                    await Unitat.insertMany(ar_Records)
                        .then(
                            (docs) =>
                                docs?.length &&
                                docs.map((doc) =>
                                    console.info(
                                        ` - IMEI: ${
                                            doc?.properties?.imei
                                        }, ID:  ${
                                            doc?.properties?.event_id
                                        }, HORA: ${new Date(
                                            doc?.properties?.timestamp
                                        ).toLocaleString("ca-ES", {
                                            timeZone: "Europe/Madrid",
                                        })}, COOR (Lat/Long): ${
                                            doc?.geometry?.coordinates[1]
                                        } ${doc?.geometry?.coordinates[0]}`
                                    )
                                )
                        )
                        .catch((e) => console.error(e));
                    console.log("Tancant sessió");
                }
            });
            socket.on("timeout", function (error) {
                console.error("Socket timeout");
                enviarLog({
                    type: "error",
                    source: "socket_time_out",
                    data: error,
                });
                socket.destroy();
            });
            socket.on("end", () => {
                console.log("client disconnected");
            });
            socket.on("error", (err) => {
                console.error(err);
                // throw err;
                enviarLog({
                    type: "error",
                    source: "socket_conn_dongle",
                    data: err,
                });
                socket.destroy();
            });
        });

        this.server.on("error", (err) => {
            console.error(err);
            //throw err;
            enviarLog({
                type: "error",
                source: "server_tcp",
                data: err,
            });
            this.server.close();
            listen();
        });
        this.server.on("uncaughtException", (err) => {
            console.error(err);
            //throw err;
            enviarLog({
                type: "uncaughtException",
                source: "server_tcp",
                data: err,
            });
            this.server.close();
            listen();
        });
    }
}
module.exports = Server;
