const { enviarLog } = require("../models/log_server");
const Parser = require("teltonika-parser-extended");
const { getUnitByData, Unitat } = require("../models/unitat");
const binutils = require("binutils64");
const fs = require("fs");
const tls = require("tls");

class Server {
    constructor() {
        try {
            const options = {
                key: fs.readFileSync(
                    "sslcert/server_teltonika/server.key",
                    "utf8"
                ),
                cert: fs.readFileSync(
                    "sslcert/server_teltonika/server.crt",
                    "utf8"
                ),

                // This is necessary only if using client certificate authentication.
                requestCert: true,
                rejectUnauthorized: false,

                //This is necessary only if the client uses a self-signed certificate.
                ca: [
                    fs.readFileSync(
                        "sslcert/server_teltonika/root.pem",
                        "utf8"
                    ),
                ],
            };

            this.server = tls.createServer(options);

            this.port = process.env.PORT_DEVICES_SSL;

            // // Rutas de mi aplicación
            this.routes();
        } catch (e) {
            console.error(e);
            enviarLog({
                type: "error",
                source: "constructor_server_SSL_dongle",
                message: JSON.stringify(e),
            });
        }
    }

    listen() {
        this.server.listen(this.port, () => {
            console.log(
                `Servidor Teltonika SSL escoltant al port ${this.port}`
            );
        });
    }

    routes() {
        this.server.on("connection", async (socket) => {
            // console.log("Connexió no segura");
        });

        this.server.on("secureConnection", function (socket) {
            // socket.authorized will be true if the client cert presented validates with our CA
            // console.log(
            //     "Secure connection. Client authorized: " + socket.authorized
            // );

            let imei;
            // console.log("Client SSL connectat");
            // console.log(
            //     "Ip del client: " +
            //         socket.remoteAddress +
            //         ":" +
            //         socket.remotePort
            // );

            socket.on("data", async (data) => {
                // console.log("Llegint dades encriptades del GPS");

                let buffer = data;
                let parser = new Parser(buffer);

                if (parser.isImei && parser?.imei?.length) {
                    socket.write(Buffer.alloc(1, 1));
                    imei = parser.imei;
                    // console.log(`ENTRADA IMEI SSL: ${parser.imei}`);
                } else if (imei?.length > 0) {
                    let avl = parser.getAvl();
                    let writer = new binutils.BinaryWriter();
                    writer.WriteInt32(avl.number_of_data);

                    let response = writer.ByteBuffer;
                    socket.write(response);

                    // TODO parser
                    let ar_Records = [];
                    if (
                        avl.hasOwnProperty("records") &&
                        Array.isArray(avl?.records) &&
                        avl.records.length > 0
                    ) {
                        for (let d of avl.records) {
                            d.imei = imei;
                            ar_Records.push(getUnitByData(d));
                        }
                    }
                    // await Unitat.init();
                    await Unitat.insertMany(ar_Records)
                        .then(
                            (docs) =>
                                docs?.length &&
                                docs.map((doc) =>
                                    console.info(
                                        ` - IMEI SSL: ${
                                            doc?.properties?.imei
                                        }, ID:  ${
                                            doc?.propergities?.event_id
                                        }, HORA: ${new Date(
                                            doc?.properties?.timestamp
                                        ).toLocaleString("ca-ES", {
                                            timeZone: "Europe/Madrid",
                                        })}, COOR (Lat/Long): ${
                                            doc?.geometry?.coordinates[1]
                                        } ${doc?.geometry?.coordinates[0]}`
                                    )
                                )
                        )
                        .catch((e) => console.error(e));
                    // console.log("Tancant sessió SSL");
                } else {
                    console.info("CONEXIÓ TANCADA");
                    console.log(
                        "Ip del client sense IMEI: " +
                            socket.remoteAddress +
                            ":" +
                            socket.remotePort
                    );
                    socket.destroy();
                }
            });

            socket.on("timeout", function (error) {
                console.error("Socket timeout");
                enviarLog({
                    type: "error",
                    source: "socket_SSL_time_out",
                    data: error,
                });
                socket.destroy();
            });

            socket.on("end", () => {
                // console.log("client SSL disconnected");
            });

            socket.on("error", (err) => {
                console.error(err);
                // throw err;
                enviarLog({
                    type: "error",
                    source: "socket_SSL_conn_dongle",
                    data: err,
                });
                socket.destroy();
            });
        });

        this.server.on("error", (err) => {
            console.error(err);
            //throw err;
            enviarLog({
                type: "error",
                source: "server_SSL_tcp",
                data: err,
            });
            this.server.close();
            listen();
        });

        this.server.on("uncaughtException", (err) => {
            console.error(err);
            //throw err;
            enviarLog({
                type: "uncaughtException",
                source: "server_SSL_tcp",
                data: err,
            });
            this.server.close();
            listen();
        });
    }
}
module.exports = Server;
